local myUI = ...

local tinsert = table.insert
local slen = string.len
local ssub = string.sub
local sgsub = string.gsub
local srep = string.rep
local sfind = string.find
local mfloor = math.floor

local function _ifnil(test, replacement)
	if (test == nil) then
		return replacement
	else
		return test
	end
end

	
local function _isnil(test) return (test == nil) end
	
local function _notnil(test) return (test ~= nil) end
	
local function _iif(test, truevalue, falsevalue)
	if (test) then
		return truevalue
	else
		return falsevalue
	end
end
	
local function _shallowCopy(t)
	if (t == nil) then
		return nil
	elseif (type(t) ~= "table") then
		return t
	else
		local r = {}
		for k, v in pairs(t) do
			if (type(v) == "table") then
				r[k] = _shallowCopy(v)
			else
				r[k] = v
			end
		end
		return r
	end
end

local function _nilCopy(dest, defs)
	if (dest ~= nil) and (defs ~= nil) then
		if (type(dest) == "table") and (type(defs == "table")) then
			for k, v in pairs(defs) do
				if (type(v) == "table") then
					if (dest[k] == nil) then
						dest[k] = {}
					end
					
					_nilCopy(dest[k], v)
				else
					if (dest[k] == nil)  then
						dest[k] = v
					end
				end
			end
		end
	end
end
	
local function _trim(s) return  sgsub(s, "^%s*(.-)%s*$", "%1") end
	
local function  _ltrim(s) return (sgsub(s, "^%s*", "")) end
local function _rtrim(s)
    local n = #s
    while n > 0 and s:find("^%s", n) do n = n - 1 end
    return s:sub(1, n)
end

local function _lpad(s, len, pchar)
    pchar = _ifnil(pchar, " ");
    if (slen(pchar) > 1) then pchar = ssub(pchar,1,1); end

    s = srep(pchar, len) .. s;    
    return ssub(s, -len);
end

local function _rpad(s, len, pchar)
    pchar = _ifnil(pchar, " ");
    if (slen(pchar) > 1) then pchar = ssub(pchar,1,1); end

    s = s .. srep(pchar, len);
    return ssub(s, 1, len);
end
   
local function _tohex(n)
    if (n ~= nil) then
        n = tonumber(n);        
        return _lpad(string.format("%x", n), 2, "0");
    else
        return "nil";
    end
end
	
local function _nilstr(s) return tostring(_ifnil(s, "nil")) end

local function _fromCSV(s)
	s = s .. ','        -- ending comma
	local t = {}        -- table to collect fields
	local fieldstart = 1
	repeat
	-- next field is quoted? (start with `"'?)
	if sfind(s, '^"', fieldstart) then
	local a, c
	local i  = fieldstart
	repeat
	    -- find closing quote
	    a, i, c = sfind(s, '"("?)', i+1)
	until c ~= '"'    -- quote not followed by quote?
	if not i then error('unmatched "') end
	local f = ssub(s, fieldstart+1, i-1)
	tinsert(t, (sgsub(f, '""', '"')))
	fieldstart = sfind(s, ',', i) + 1
	else                -- unquoted; find next comma
	local nexti = sfind(s, ',', fieldstart)
	tinsert(t, ssub(s, fieldstart, nexti-1))
	fieldstart = nexti + 1
	end
	until fieldstart > slen(s)
	return t
end

local function _isempty(s)
    if (nil == s) then
        return true;
    elseif (type(s) == "string") then            
        return not (string.len(_trim(s)) > 0);            
    else
        return not (string.len(tostring(s)) > 0);
    end                     
end

local function _ifempty(s, replacement)
	if (nil == s) or (s == "") then
		return replacement
	else
		return s
	end
end	
	
local function _lpad(s, len, pchar)
	pchar = _ifnil(pchar, " ");
	if (slen(pchar) > 1) then pchar = ssub(pchar,1,1); end

	s = srep(pchar, len) .. s;    
	return ssub(s, -len);
end
	
local function _minmax(value, min, max)
	if (value < min) then
		value = min
	end
	if (value > max) then
		value = max
	end
	
	return value
end

local function _round(num, idp)
	local mult = 10^(idp or 0)
	return mfloor(num * mult + 0.5) / mult
end
	
local function _pprint(...)
	local msg = ""
	local n = select('#', ...)
	for i = 1, n do
		local v = select(i, ...)
		if (v == nil) then
			v = "nil"
		else
			v = tostring(v)
		end
		msg = msg .. v
	end

	print(msg)
end
	
local function _rgba(c) 
	if (c.r) then
		return c.r, c.g, c.b, c.a 
	else
		return c[1], c[2], c[3], c[4]
	end
end

local function _parseNumber(value)
	local r = false; local n = nil
	
	if (value ~= nil) then
		value = sgsub(tostring(value), "^%s*(.-)%s*$","%1")
		r = pcall(function() n = tonumber(value) end)
	end
	
	return r, n
end



myUI.Utility = {
	ifnil = _ifnil,
	isnil = _isnil, 
	notnil = _notnil,
	iif = _iif,
	shallowCopy = _shallowCopy,
	nilCopy = _nilCopy,
	trim = _trim, 
	rtrim = _rtrim,
	ltrim = _ltrim,
	lpad = _lpad,
	rpad = _rpad,
	tohex = _tohex,
	nilstr = _nilstr,
	isempty = _isempty,
	ifempty = _ifempty,
	fromCSV = _fromCSV,
	minmax = _minmax,
	round = _round,
	pprint = _pprint,
	rgba = _rgba,
	parseNumber = _parseNumber,
	
}